using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    //! list of (float-value,identifier) features
    class FeatureVector : List<float>
    {
        int m_nIdentifier;
        public int Identifier {
            get { return m_nIdentifier; }
            set { m_nIdentifier = value; }
        }

        public FeatureVector(int nIdentifier) {
            m_nIdentifier = nIdentifier;
        }

        public float MaxValue {
            get { 
                if ( this.Count < 1 )
                    return float.MinValue;
                int iMax = 0;
                for ( int i = 1; i < this.Count; ++i )
                    iMax = (this[i] > this[iMax]) ? i : iMax;
                return this[iMax];
            }
        }

        public float AverageValue {
            get {
                float fAverage = 0.0f;
                for ( int i = 0; i < this.Count; ++i )
                    fAverage += this[i];
                return fAverage / (float)this.Count;
            }
        }
    }


    //! set of FeatureVector objects, can be treated as a long feature vector
    class FeatureSet : List<FeatureVector>
    {
        public FeatureSet() {
        }

        public FeatureVector MaxValueFeature {
            get { 
                float fMax = float.MinValue;
                int iMax = -1;
                for ( int i = 0; i < this.Count; ++i ) {
                    float fCur = this[i].MaxValue;
                    if ( fCur > fMax ) {
                        fMax = fCur;
                        iMax = i;
                    }
                }
                return (iMax >= 0) ? this[iMax] : null;
            }
        }


        public FeatureVector MaxAverageValueList {
            get { 
                float fMaxAverage = float.MinValue; 
                int iMax = -1;
                for ( int i = 0; i < this.Count; ++i ) {
                    float fAverage = this[i].AverageValue;
                    if ( fAverage > fMaxAverage ) {
                        fMaxAverage = fAverage;
                        iMax = i;
                    }
                }
                return (iMax >= 0) ? this[iMax] : null;
            }
        }
    }


}
