using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    class TransitionBehaviorManager
    {
        List<TransitionBehavior> m_targets;
        Dictionary<Renderable, TransitionBehavior> m_defaults;

        public void Register(TransitionBehavior behavior) {
            if ( ! m_targets.Contains( behavior ) )
                m_targets.Add(behavior);
        }
        public void DeRegister(TransitionBehavior behavior) {
            m_targets.Remove(behavior);
        }

        public TransitionBehavior GetDefaultTransition(Renderable target) {
            if (!m_defaults.ContainsKey(target))
                m_defaults[target] = new FadeInOutBehavior(target);
            return m_defaults[target];
        }


        public void Hide( BehaviorTarget target ) {
            Renderable renderable = target as Renderable;
            if ( renderable != null && Element.VisibleForLayout(renderable.Visibility) ) {

                // either fire transition behaviors for item, or run a default behavior (currently a fade)

                if (target.Behaviors.TransitionBehaviors.Count > 0) {
                    foreach (TransitionBehavior trans in target.Behaviors.TransitionBehaviors)
                        trans.Stop();
                    foreach (TransitionBehavior trans in target.Behaviors.TransitionBehaviors)
                        trans.OnHide();
                } else {
                    GetDefaultTransition(renderable).Stop();
                    GetDefaultTransition(renderable).OnHide();
                }
            }
        }

        public void Show( BehaviorTarget target ) {
            Renderable renderable = target as Renderable;
            if ( renderable != null && ! Element.VisibleForLayout(renderable.Visibility) ) {

                // either fire transition behaviors for item, or run a default behavior (currently a fade)

                if (target.Behaviors.TransitionBehaviors.Count > 0) {
                    foreach (TransitionBehavior trans in target.Behaviors.TransitionBehaviors)
                        trans.Stop();
                    foreach (TransitionBehavior trans in target.Behaviors.TransitionBehaviors)
                        trans.OnShow();
                } else {
                    GetDefaultTransition(renderable).Stop();
                    GetDefaultTransition(renderable).OnShow();
                }
                    
            }
        }


        static public TransitionBehaviorManager Get {
            get {
                if (s_singleton == null)
                    s_singleton = new TransitionBehaviorManager();
                return s_singleton;
            }
        }

        protected TransitionBehaviorManager() {
            m_targets = new List<TransitionBehavior>();
            m_defaults = new Dictionary<Renderable, TransitionBehavior>();
        }
        static TransitionBehaviorManager s_singleton = null;
    }
}

