using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    class VerticalAlignedRowLayout : Layout
    {
        public enum AlignMode {
            Left,
            Right,
            Center
        };

        Box2f m_bounds;
        AlignMode m_eAlignMode;
        List<Item> m_vNewlineItems;

        public VerticalAlignedRowLayout() {
            m_bounds = new Box2f(false);
            m_vNewlineItems = new List<Item>();
        }

        public AlignMode Align {
            get { return m_eAlignMode; }
            set { m_eAlignMode = value; }
        }


#region  Layout interface

        protected override Box2f ItemBounds {
            get { return m_bounds; }
        }

        public override void AppendHint(LayoutHint hint) {
            if ( hint is NewlineHint ) {
                m_vNewlineItems.Add( ((NewlineHint)hint).After );
            } 
        }


        protected override void LayoutObjects(Container c) {
            Box2f childbounds = c.ChildAreaBounds;
            float fWidth = childbounds.Width;

            // compute row widths
            List<float> vRowWidths = new List<float>();
            float fMaxWidth = 0;
            float fRowWidth = 0;
            for ( int i = 0; i < c.Items.Count; ++i ) {
                fRowWidth += c.Items[i].Bounds.Width;
                if ( m_vNewlineItems.Contains(c.Items[i]) ) {
                    vRowWidths.Add(fRowWidth);
                    fMaxWidth = (fRowWidth > fMaxWidth) ? fRowWidth : fMaxWidth;
                    fRowWidth = 0;
                }
            }

            // convert row widths to offsets
            for ( int i = 0; i < vRowWidths.Count; ++i ) {
                switch ( m_eAlignMode ) {
                    case AlignMode.Left:
                        vRowWidths[i] = 0;
                        break;
                    case AlignMode.Center:
                        vRowWidths[i] = (fWidth - vRowWidths[i]) * 0.5f;
                        break;
                    case AlignMode.Right:
                        break;
                }
            }


            int nRow = 0;
            float fRowHeight = 0;
            float fCurY = 0;
            float fCurX = vRowWidths[nRow];
            for (int i = 0; i < c.Items.Count; ++i) {
                Item curItem = c.Items[i];
                Box2f itemsize = curItem.Bounds;

                // update item position wrt top of container
                Box2f itemBounds = curItem.Size.ToBox();
//                itemBounds.MoveMin(childbounds.Min.X + fCurX, childbounds.Min.Y + fCurY);
                itemBounds.MoveMin(fCurX, fCurY);
                itemBounds = c.MoveRelativeToTop(itemBounds);

                curItem.Resize(itemBounds);

                fCurX += itemsize.Width;
                if (itemsize.Height > fRowHeight)
                    fRowHeight = itemsize.Height;

                if (i == 0)
                    m_bounds = curItem.Bounds;
                else
                    m_bounds.Contain(curItem.Bounds);

                if ( m_vNewlineItems.Contains(curItem) ) {
                    fCurY += fRowHeight;
                    ++nRow;
                    fCurX = (nRow < vRowWidths.Count) ? vRowWidths[nRow] : 0;
                    fRowHeight = 0;
                }
            }
        }

#endregion



        //! this is kind of a hack...breaks if item set changes!
        public class NewlineHint : LayoutHint {
            Item m_afterItem;

            public NewlineHint( Item afterItem ) {
                m_afterItem = afterItem;
            }

            public Item After {
                get { return m_afterItem; }
                set { m_afterItem = value; }
            }
        }
    }
}
