using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    class VerticalStackingExpanderLayout : Layout
    {
        Box2f m_bounds;

        public VerticalStackingExpanderLayout(Box2f bounds) {
            m_bounds = bounds;
        }

 #region  Layout interface

       protected override Box2f ItemBounds {
            get { return m_bounds; }
        }

        protected override void LayoutObjects(Container c) {
            m_bounds = c.ChildAreaBounds;
            Box2f childbounds = m_bounds;

            // find total height of objects
            float fTotalHeight = 0;
            float fFixedHeight = 0;
            float fScalableHeight = 0;
            for (int i = 0; i < c.Items.Count; ++i) {
                if ( ! Element.VisibleForLayout(c.Items[i].Visibility))
                    continue;

                float fHeight = c.Items[i].Bounds.Height;
                fTotalHeight += fHeight;
                if ( c.Items[i].CanResizeHeight )
                    fScalableHeight += fHeight;
                else
                    fFixedHeight += fHeight;
            }

            // figure out how much to scale scalable items
            float fCurrentHeight = fTotalHeight - fFixedHeight;
            float fAvailableHeight = childbounds.Height - fFixedHeight;
            float fScale = 1 - (fCurrentHeight-fAvailableHeight) / fCurrentHeight;

            // ok, now add all child items. Don't resize the first one, but resize the
            // rest so that the total height adds up
            float fCurY = 0;
            float fWidth = childbounds.Width;
            for (int i = 0; i < c.Items.Count; ++i) {
                if (!Element.VisibleForLayout(c.Items[i].Visibility))
                    continue;

                Box2f itemBounds = c.Items[i].Size.ToBox();

                float fNewHeight = itemBounds.Height;
                if ( c.Items[i].CanResizeHeight )
                    fNewHeight *= fScale;

                itemBounds.SetWidth(fWidth, Box2f.ScaleMode.ScaleRight);
                itemBounds.SetHeight(fNewHeight, Box2f.ScaleMode.ScaleDown);
                itemBounds.MoveMin(0, fCurY);
                itemBounds = c.MoveRelativeToTop(itemBounds);

                c.Items[i].Resize(itemBounds);

                fCurY += fNewHeight;
            }
        }

#endregion 

    }
}
