using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher {

    class WidgetContainerBridge : FixedRectContainer {

        Widget m_widget;

        public WidgetContainerBridge(Widget widget) : base(widget.Bounds, new VerticalStackingExpanderLayout(widget.Bounds)) 
        {
            m_widget = widget;

            BackgroundColor = ColorRGBA.WHITE;
            ForegroundColor = ColorRGBA.WHITE;
        }


        public Widget ParentWidget {
            get { return m_widget; }
            set { m_widget = value; }
        }


        override protected Box2f GetChildAreaRegion() {
            return m_widget.Bounds;
        }


        override public VisibilityState Visibility {
            get {
                if (ParentWidget != null)
                    return Element.DetermineVisibility(m_Visibility, ParentWidget.Visibility);
                return m_Visibility;
            }
        }

        override public float Opacity {
            get { return (ParentWidget == null) ? m_fOpacity : m_fOpacity * ParentWidget.Opacity; }
        }



#region Item Overrides

        override public void Render( ) {
            ValidateLayout();

            // set clip region for child
            RenderEngine.Get.PushClipBox( ChildAreaBounds );

            for (int i = 0; i < Items.Count; ++i) {
                if (Element.VisibleForRendering( Items[i].Visibility ))
                    Items[i].Render();
            }

            RenderEngine.Get.PopClipBox();
        }
#endregion

    }
}
