using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    class CrossKeyDisc : RadialCrossMenu
    {
        Image m_discInterior;
        Image m_discShadow;
        float m_fDropShadowWidth;

        public CrossKeyDisc(Element parent, float fRadius) : base(parent, fRadius) {
            m_fDropShadowWidth = 20.0f;

            int nTexSize = 128;

            float fInnerRad = 0.25f;
            m_discInterior = new Image(nTexSize, nTexSize);
            Image.Pixel pix = new Image.Pixel(0.75f, 0.75f, 1.0f);
            for ( int yi = 0; yi < nTexSize; ++yi ) {
                for ( int xi = 0; xi < nTexSize; ++xi ) {
                    float x = ((float)xi / (float)nTexSize - 0.5f) * 2.0f;
                    float y = ((float)yi / (float)nTexSize - 0.5f) * 2.0f;
                    float fDist = (float)Math.Sqrt(x*x + y*y);
                    pix.fA = MathUtil.WyvillFalloff(fDist, fInnerRad, 1);
                    m_discInterior.SetPixel(xi, yi, pix);
                }
            }

            m_discShadow = new Image(nTexSize, nTexSize);
            pix = new Image.Pixel(0.0f, 0.0f, 0.0f);
            fInnerRad = 0.5f;
            for ( int yi = 0; yi < nTexSize; ++yi ) {
                for ( int xi = 0; xi < nTexSize; ++xi ) {
                    float x = ((float)xi / (float)nTexSize - 0.5f) * 2.0f;
                    float y = ((float)yi / (float)nTexSize - 0.5f) * 2.0f;
                    float fDist = (float)Math.Sqrt(x*x + y*y);
                    pix.fA = MathUtil.WyvillFalloff(fDist, fInnerRad, 1);
                    m_discShadow.SetPixel(xi, yi, pix);
                }
            }
        }


        public float DropShadowWidth {
            get { return m_fDropShadowWidth; }
            set { m_fDropShadowWidth = value; }
        }


        override protected void RenderWidgetBackground() {
            // drop shadow
            Box2f shadowBounds = new Box2f(m_circle.Center, m_circle.Radius + m_fDropShadowWidth);
            RenderEngine.Get.DrawImage( m_discShadow, shadowBounds );
        }

        override protected void RenderWidget() {
            ColorRGBA cInterior = ColorRGBA.WHITE;
            cInterior.A *= Opacity;

            // white background
            RenderEngine.Get.DrawCircle(m_circle.Center, m_circle.Radius, cInterior);

            //ColorRGBA cInnerBorder = new ColorRGBA(0.5f, 0.5f, 0.5f);
            //RenderEngine.Get.DrawCircle(m_circle.Center, 0.45f * m_circle.Radius, cInnerBorder);
            ColorRGBA cInner = new ColorRGBA(0.75f, 0.75f, 1.0f);
            //RenderEngine.Get.DrawCircle(m_circle.Center, 0.4f * m_circle.Radius, cInner);

            Box2f bounds = new Box2f(m_circle.Center, m_circle.Radius );
            RenderEngine.Get.DrawImage( m_discInterior, bounds );
        }
    }
}
