using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.Runtime.InteropServices;

using Tao.OpenGl;
using Tao.Platform.Windows;

using VBTablet;

namespace Sketcher
{
    public partial class OpenGLTestForm : Form
    {
        [DllImport( "user32.dll", EntryPoint=("GetSystemMetrics") )]
        public static extern int GetSystemMetrics( int nIndex );
        const int SM_XVIRTUALSCREEN = 76;
        const int SM_YVIRTUALSCREEN = 77;
        const int SM_CXVIRTUALSCREEN = 78;
        const int SM_CYVIRTUALSCREEN = 79;

        [StructLayout( LayoutKind.Sequential )]
        public struct RECT {
            public int left;
            public int top;
            public int right;
            public int bottom;
        }
        [DllImport( "user32.dll", SetLastError=true )]
        static extern bool GetWindowRect( IntPtr hWnd, out RECT lpRect ); 



        List<Stroke> m_vStrokes;
        bool m_bInStroke;

        Tablet m_tablet;
        bool m_bUseSubPixelAccuracy;
        int m_nMaxTabletPressure;
        int m_nTabletCoordsYHeight;
        int[] m_nCVirtualScreenSize = {0,0};
        bool m_bTabletActive;
        bool m_bTabletDown;


        struct TabPoint {
            public float fX;
            public float fY;
            public float fPressure;
        };
        TabPoint[] m_queue;
        int m_nQueueSize;


        GLCanvas m_glcanvas;

        public OpenGLTestForm()
        {
            InitializeComponent();
            glCanvas1.TaoRenderEnabled = true;

            m_vStrokes = new List<Stroke>();
            m_bInStroke = false;

            m_nQueueSize = 0;
            InitializeTablet();

            m_glcanvas = new GLCanvas(glCanvas1);
        }

        void InitializeTablet() {
            try {
                m_tablet = new Tablet();
            } catch (Exception) {
                return;
            }

            m_tablet.hWnd = this.Handle;
            string contextID = "tabletContext";
            bool bIsDigitizing = false;
            contextID = m_tablet.AddContext(contextID, ref bIsDigitizing);
            m_tablet.SelectContext(ref contextID);
            m_tablet.hWnd = this.Handle;
            m_tablet.Connected = true;

            bool bEnable = true;
            m_tablet.Context.OpenContext(ref bEnable);
            m_tablet.Context.QueueSize = 64;

            m_bUseSubPixelAccuracy = true;
            if (m_bUseSubPixelAccuracy) {
                m_nCVirtualScreenSize[0] = GetSystemMetrics( SM_CXVIRTUALSCREEN );
                m_nCVirtualScreenSize[1] = GetSystemMetrics( SM_CYVIRTUALSCREEN );
                m_tablet.Context.OutputOriginX = 0;
                m_tablet.Context.OutputExtentX = m_nCVirtualScreenSize[0] << 4;
                m_tablet.Context.OutputOriginY = 0;
                m_tablet.Context.OutputExtentY = m_nCVirtualScreenSize[1] << 4;
            }

            m_tablet.PacketArrival += new Tablet.PacketArrivalEventHandler(Tablet_Packet);
            m_tablet.ProximityChange += new Tablet.ProximityChangeEventHandler(Tablet_ProximityChange);

            m_tablet.Context.Update();

            // [RMS] hack to get max pressure value for tablet
            modDeclarations.tagAxis axis = new modDeclarations.tagAxis();
            modDeclarations.WTInfo(99, modDeclarations.DVC_NPRESSURE, ref axis);
            m_nMaxTabletPressure = axis.Max;

            m_nTabletCoordsYHeight = m_tablet.Context.OutputExtentY;

            m_queue = new TabPoint[32];
            m_nQueueSize = 0;

            m_bTabletDown = false;
            m_bTabletActive = false;
        }


        public void Tablet_ProximityChange(ref bool bInContext, ref bool bIsPhysical, ref IntPtr contextHandle, ref string contextName)
        {
            // [RMS] it may be necessary to start a short input-event timeout here. On some older Wacoms the 
            //   fake 'mouse events' we ignored based on m_bTabletActive sometimes arrive after the proximity change event.
            //   This is irritating.
            m_bTabletActive = bInContext;
        }

        public void Tablet_Packet(ref System.IntPtr ContextHandle, 
            ref int Cursor, ref int X, ref int Y, ref int Z, ref int Buttons, 
            ref int NormalPressure, ref int TangentPressure, ref int Azimuth, 
            ref int Altitude, ref int Twist, ref int Pitch, ref int Roll, 
            ref int Yaw, ref int PacketSerial, ref int PacketTime)
        {
            //Debug.WriteLine(ContextHandle.ToString() + "  " + Cursor + "  " + (float)NormalPressure / (float)m_nMaxTabletPressure + "   (" + X + "," + Y + ")");

            lock (m_queue) {

                float fX, fY;
                if (m_bUseSubPixelAccuracy) {
                    RECT rcWindow = new RECT();
                    GetWindowRect( glCanvas1.Handle, out rcWindow );
                    RECT rcScreen = new RECT();
                    rcScreen.left = GetSystemMetrics( SM_XVIRTUALSCREEN );
                    rcScreen.top = GetSystemMetrics( SM_YVIRTUALSCREEN );
                    rcScreen.bottom = rcScreen.top + GetSystemMetrics( SM_CYVIRTUALSCREEN );
                    fX = (float)(rcScreen.left - rcWindow.left) + (float)X / 16.0f;
                    fY = (float)(rcWindow.bottom - rcScreen.bottom) + (float)Y / 16.0f;

                } else {
                    Point e = glCanvas1.PointToClient( new Point( X, m_nTabletCoordsYHeight - 1 - Y ) );
                    fX = (float)e.X;
                    fY = (float)glCanvas1.Height - 1 - (float)e.Y;
                }

                if (NormalPressure > 0) {
                    if (m_nQueueSize < m_queue.Length - 1) {
                        m_queue[m_nQueueSize].fX = fX;
                        m_queue[m_nQueueSize].fY = fY;
                        m_queue[m_nQueueSize].fPressure = (float)NormalPressure / (float)m_nMaxTabletPressure;
                        m_nQueueSize++;
                    }
                    m_bTabletDown = true;
                    AppendTabletQueue(false);
                } else {
                    if (m_bTabletDown && m_bInStroke) {
                        AppendTabletQueue(true);
                        Stroke_End();
                        m_nQueueSize = 0;
                    } else {
                        m_glcanvas.OnHover( new Vector2f( fX, fY ) );
                    }
                    m_bTabletDown = false;
                }
            }
        }




        private void AppendTabletQueue(bool bForce) {

            lock (m_queue) {
                if (m_nQueueSize == 0)
                    return;
                //System.Diagnostics.Debug.WriteLine("Queue size is " + m_nQueueSize);

                int i = 0;
                if (!m_bInStroke) {
                    Stroke_Start( m_queue[0].fX, m_queue[0].fY, m_queue[0].fPressure );
                    i++;

                    Begin_Fast_Stroke_Append();
                    for (; i < m_nQueueSize; ++i)
                        Fast_Stroke_Append(m_queue[i].fX, m_queue[i].fY, m_queue[i].fPressure);
                    Done_Fast_Stroke_Append();
                    m_nQueueSize = 0;

                } else if (m_nQueueSize > 2 || bForce) {
                    Begin_Fast_Stroke_Append();
                    for (; i < m_nQueueSize; ++i)
                        Fast_Stroke_Append(m_queue[i].fX, m_queue[i].fY, m_queue[i].fPressure);
                    Done_Fast_Stroke_Append();
                    m_nQueueSize = 0;
                }
            }
        }



        private void glCanvas1_MouseMove(object sender, MouseEventArgs e) {
            if (m_bTabletActive)
                return;
            if (m_bInStroke) {
                Begin_Fast_Stroke_Append();
                Fast_Stroke_Append( e.X, (float)glCanvas1.Height - 1 - e.Y, 1.0f );
                Done_Fast_Stroke_Append();
            } else {
                m_glcanvas.OnHover( new Vector2f( e.X, glCanvas1.Height - 1 - e.Y ) );
            }
        }


        private void glCanvas1_MouseDown(object sender, MouseEventArgs e) {
            if (m_bTabletActive)
                return;
            Stroke_Start( e.X, (float)glCanvas1.Height - 1 - e.Y, 1.0f );
        }

        private void glCanvas1_MouseUp(object sender, MouseEventArgs e) {
            if (m_bTabletActive)
                return;
            Stroke_End();
        }



        private void Stroke_Start(float fX, float fY, float fPressure) {
            Stroke s = new Stroke();
            s.AddVertex(fX, fY, fPressure);
            m_vStrokes.Add(s);
            m_bInStroke = true;

            m_glcanvas.OnStrokeBegin(m_vStrokes[m_vStrokes.Count - 1]);
        }

        private void Stroke_End() {
            if (m_bInStroke) {
                m_bInStroke = false;

                Debug.WriteLine("Stroke size: " + m_vStrokes[m_vStrokes.Count - 1].Vertices.Count);
                m_glcanvas.OnStrokeEnd(m_vStrokes[m_vStrokes.Count - 1]);

                m_vStrokes.Clear();

                glCanvas1.Invalidate();
            }
        }


        private void Begin_Fast_Stroke_Append() {
            if (!m_bInStroke)
                return;
        }


        private void Fast_Stroke_Append(float fX, float fY, float fPressure) {
            if (!m_bInStroke)
                return;
            Stroke curStroke = m_vStrokes[m_vStrokes.Count - 1];
            curStroke.AddVertex(fX, fY, fPressure);
        }


        private void Done_Fast_Stroke_Append() {
            if (!m_bInStroke)
                return;
            Stroke curStroke = m_vStrokes[m_vStrokes.Count - 1];
            m_glcanvas.OnStrokeUpdate(curStroke);
        }





        private void glCanvas1_TaoSetupContext(object sender, EventArgs e)
        {
            m_glcanvas.Initialize();

            // initialize scene
            m_glcanvas.InitializeTest();
            AnimationTimer.Interval = 10;
            AnimationTimer.Start();
        }

        private void glCanvas1_TaoRenderScene(object sender, EventArgs e)
        {
            m_glcanvas.Render();
            Wgl.wglSwapBuffers(Wgl.wglGetCurrentDC());

            this.Text = m_glcanvas.ApplicationTitle;
        }

        private void OpenGLTestForm_Resize(object sender, EventArgs e) {
            m_glcanvas.Resized(glCanvas1.Width, glCanvas1.Height);
        }

        private void AnimationTimer_Tick(object sender, EventArgs e) {
            if ( m_glcanvas != null )
                m_glcanvas.AnimationTick();
        }

    }
}