using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    struct Vector2f
    {
        public float X;
        public float Y;


        public static Vector2f ZERO = new Vector2f(0, 0);
        public static Vector2f UP = new Vector2f(0, 1);
        public static Vector2f DOWN = new Vector2f(0, -1);
        public static Vector2f LEFT = new Vector2f(0, -1);
        public static Vector2f RIGHT = new Vector2f(0, 1);


        public Vector2f(float x, float y) {
            X = x; Y = y;
        }

        public Vector2f(Vector2f o) {
            X = o.X; Y = o.Y;
        }

        public float Dot(Vector2f o) {
            return X * o.X + Y * o.Y;
        }

        public float Cross(Vector2f o) {
            return X * o.Y - Y * o.X;
        }

        public float Length {
            get { return (float)Math.Sqrt(X * X + Y * Y); }
        }
        public float SquaredLength {
            get { return X * X + Y * Y; }
        }

        public float Normalize() {
            float fLen = (float)Math.Sqrt(X * X + Y * Y);
            if (fLen > 0) {
                X /= fLen;
                Y /= fLen;
            }
            return fLen;
        }
        public Vector2f Normalized() {
            Vector2f v = this;
            v.Normalize();
            return v;
        }

        public float SquaredDist(Vector2f o) {
            return ((X - o.X) * (X - o.X) + (Y - o.Y) * (Y - o.Y));
        }
        public float SquaredDist( float fX, float fY ) {
            return ((X - fX) * (X - fX) + (Y - fY) * (Y - fY));
        }
        public float Dist(Vector2f o) {
            return (float)Math.Sqrt((X - o.X) * (X - o.X) + (Y - o.Y) * (Y - o.Y));
        }



        public void Set(Vector2f o) {
            X = o.X; Y = o.Y;
        }
        public void Set(float fX, float fY) {
            X = fX; Y = fY;
        }
        public void Add(Vector2f o) {
            X += o.X; Y += o.Y;
        }
        public void Subtract(Vector2f o) {
            X -= o.X; Y -= o.Y;
        }

        public static Vector2f operator+( Vector2f a, Vector2f o ) {
            return new Vector2f(a.X + o.X, a.Y + o.Y); 
        }
        public static Vector2f operator +(Vector2f a, float f) {
            return new Vector2f(a.X + f, a.Y + f);
        }

        public static Vector2f operator-(Vector2f a, Vector2f o) {
            return new Vector2f(a.X - o.X, a.Y - o.Y);
        }
        public static Vector2f operator -(Vector2f a, float f) {
            return new Vector2f(a.X - f, a.Y - f);
        }

        public static Vector2f operator *(Vector2f a, float f) {
            return new Vector2f(a.X * f, a.Y * f);
        }
        public static Vector2f operator *(float f, Vector2f a) {
            return new Vector2f(a.X * f, a.Y * f);
        }

        

    }
}
