using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace Sketcher {
    class NotebookPage : VectorCanvas 
    {
        // [RMS] does page need to know about these? We should store them
        //   in Notebook (at least the toolbar widget...)
        CameraWidget2D[] m_cameraWidgets = { null, null };
        ToolbarWidget m_toolbarWidget;

        public NotebookPage(Box2f bounds, Notebook notebook) : base(bounds) 
        {
            const float fButtonSize = 32.0f;

            ForegroundColor = ColorRGBA.WHITE;
            BackgroundColor = ColorRGBA.WHITE;

            string sResourcePath = NotebookAppConstants.IconResourcePath;

            // add cameras in bottomleft and topright
            m_cameraWidgets[0] = new CameraWidget2D(this, fButtonSize, this);
            m_cameraWidgets[0].Visibility = VisibilityState.Visible;
            m_cameraWidgets[0].DockLocation = ToolbarWidget.Location.BottomLeft;
            m_cameraWidgets[1] = new CameraWidget2D(this, fButtonSize, this);
            m_cameraWidgets[1].Visibility = VisibilityState.Visible;
            m_cameraWidgets[1].DockLocation = ToolbarWidget.Location.TopRight;

            m_toolbarWidget = new ToolbarWidget( this, fButtonSize*10, fButtonSize + 2, this );
            m_toolbarWidget.DockLocation = ToolbarWidget.Location.BottomRight;
            m_toolbarWidget.Visibility = VisibilityState.Visible;
            m_toolbarWidget.BorderWidth = 0;


            Button undoButton = new Button( fButtonSize, fButtonSize );
            undoButton.Icons.LoadBasicIconSet(
               new FilePath( FilePath.FileLocation.Resource, sResourcePath, "undo.png" ), null, null );
            undoButton.DrawBorder = false;
            undoButton.Behaviors.Add( new UndoSliderBehavior( undoButton, notebook ) );
            m_toolbarWidget.ItemContainer.Items.Add( undoButton );

            m_toolbarWidget.ItemContainer.Items.Add( new Spacer( fButtonSize/2, fButtonSize ) );

            ToggleButton<VectorToolSet> penButton = new ToggleButton<VectorToolSet>( fButtonSize, fButtonSize, Tools,
                delegate( VectorToolSet toolset ) { return toolset.SelectedToolName == VectorStrokeTool.ToolName; } );
            penButton.Icons.LoadBasicIconSet(
               new FilePath( FilePath.FileLocation.Resource, sResourcePath, "pen.png" ), null, null );
            penButton.DrawBorder = false;
            penButton.Behaviors.Add( new NotebookButtonBehavior( penButton, NotebookButtonBehavior.ButtonType.SelectPen, notebook ) );
            penButton.Behaviors.Add( new PreventCaptureBehavior( penButton ) );
            m_toolbarWidget.ItemContainer.Items.Add( penButton );


            ToggleButton<VectorToolSet> eraserButton = new ToggleButton<VectorToolSet>( fButtonSize, fButtonSize, Tools,
                delegate( VectorToolSet toolset ) { return toolset.SelectedToolName == VectorEraserTool.ToolName; } );
            eraserButton.Icons.LoadBasicIconSet(
               new FilePath( FilePath.FileLocation.Resource, sResourcePath, "eraser.png" ), null, null );
            eraserButton.DrawBorder = false;
            eraserButton.Behaviors.Add( new NotebookButtonBehavior( eraserButton, NotebookButtonBehavior.ButtonType.SelectEraser, notebook ) );
            eraserButton.Behaviors.Add( new PreventCaptureBehavior( eraserButton ) );
            m_toolbarWidget.ItemContainer.Items.Add( eraserButton );

            m_toolbarWidget.ItemContainer.Items.Add( new Spacer( fButtonSize/2, fButtonSize ) );


            Button appendPageButton = new Button( fButtonSize, fButtonSize );
            appendPageButton.Icons.LoadBasicIconSet(
                new FilePath( FilePath.FileLocation.Resource, sResourcePath, "newpage.png" ), null, null );
            appendPageButton.DrawBorder = false;
            appendPageButton.Behaviors.Add(new NotebookButtonBehavior(appendPageButton, NotebookButtonBehavior.ButtonType.AppendPage, notebook));
            appendPageButton.Behaviors.Add(new PreventCaptureBehavior(appendPageButton));
            m_toolbarWidget.ItemContainer.Items.Add(appendPageButton);

            Button prevPageButton = new Button( fButtonSize, fButtonSize );
            prevPageButton.Icons.LoadBasicIconSet(
                new FilePath( FilePath.FileLocation.Resource, sResourcePath, "prevpage.png" ), null, null );
            prevPageButton.DrawBorder = false;
            prevPageButton.Behaviors.Add(new NotebookButtonBehavior(prevPageButton, NotebookButtonBehavior.ButtonType.PreviousPage, notebook));
            prevPageButton.Behaviors.Add(new PreventCaptureBehavior(prevPageButton));
            m_toolbarWidget.ItemContainer.Items.Add(prevPageButton);

            Button nextPageButton = new Button( fButtonSize, fButtonSize );
            nextPageButton.Icons.LoadBasicIconSet(
               new FilePath( FilePath.FileLocation.Resource, sResourcePath, "nextpage.png" ), null, null );
            nextPageButton.DrawBorder = false;
            nextPageButton.Behaviors.Add(new NotebookButtonBehavior(nextPageButton, NotebookButtonBehavior.ButtonType.NextPage, notebook));
            nextPageButton.Behaviors.Add(new PreventCaptureBehavior(nextPageButton));
            m_toolbarWidget.ItemContainer.Items.Add(nextPageButton);


            m_toolbarWidget.ItemContainer.Items.Add( new Spacer( fButtonSize/2, fButtonSize ) );


            Button saveButton = new Button( fButtonSize, fButtonSize );
            saveButton.Icons.LoadBasicIconSet(
                new FilePath( FilePath.FileLocation.Resource, sResourcePath, "save.png" ), null, null );
            saveButton.DrawBorder = false;
            saveButton.Behaviors.Add( new NotebookButtonBehavior( saveButton, NotebookButtonBehavior.ButtonType.Save, notebook ) );
            saveButton.Behaviors.Add( new PreventCaptureBehavior( saveButton ) );
            m_toolbarWidget.ItemContainer.Items.Add( saveButton );

            Button openButton = new Button( fButtonSize, fButtonSize );
            openButton.Icons.LoadBasicIconSet(
                new FilePath( FilePath.FileLocation.Resource, sResourcePath, "open.png" ), null, null );
            openButton.DrawBorder = false;
            openButton.Behaviors.Add( new NotebookButtonBehavior( openButton, NotebookButtonBehavior.ButtonType.Open, notebook ) );
            openButton.Behaviors.Add( new PreventCaptureBehavior( openButton ) );
            m_toolbarWidget.ItemContainer.Items.Add( openButton );


        }





        /*
         *  save/load support
         */


        override public bool Store(XmlDocument doc, XmlElement page_node) {
            XmlElement canvas_node = doc.CreateElement("Canvas");
            page_node.AppendChild(canvas_node);
            base.Store(doc, canvas_node);

            return true;
        }


        override public bool Restore( XmlDocument doc, XmlElement page_node ) {

            XmlNodeList vCanvases = page_node.GetElementsByTagName( "Canvas" );
            if (vCanvases.Count < 1)
                return false;

            XmlElement canvas_node = vCanvases[0] as XmlElement;
            return base.Restore( doc, canvas_node );
        }

    }
}
