using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Windows.Forms;

namespace Sketcher 
{
    class NotebookButtonBehavior : ItemInputBehavior
    {
        Notebook m_notebook;

        public enum ButtonType {
            Save,
            Open,
            AppendPage,
            NextPage,
            PreviousPage,
            SelectPen,
            SelectEraser,
            Undo
        };
        ButtonType m_eType;

        public NotebookButtonBehavior(Item button, ButtonType eType, Notebook notebook)
            : base(button) {
            m_notebook = notebook;
            m_eType = eType;
        }

        //! return click "score"
        override public float GetClickScore( ClickStroke s ) {
            if (TargetItem.Geometry.IsInside( s.Click ))
                return 1.0f;
            return 0.0f;
        }

        //! handle click event
        override public void OnClick( ClickStroke s ) {

            switch (m_eType) {
                case ButtonType.Save: {
                        XmlDocument doc = new XmlDocument();
                        XmlElement notebook_node = doc.CreateElement( "Notebook" );
                        notebook_node.SetAttribute( "Version", "1" );
                        doc.AppendChild( notebook_node );
                        m_notebook.Store( doc, notebook_node );

                        SaveFileDialog saveDialog = new SaveFileDialog();
                        saveDialog.FileName = 
                            (m_notebook.Path == FilePath.UnknownPath) ? "notebook.xml" : m_notebook.Path.Filename;
                        saveDialog.Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*";
                        DialogResult result = saveDialog.ShowDialog();

                        if (result == DialogResult.OK) {
                            try { 
                                doc.Save( saveDialog.FileName );
                                m_notebook.Path = new FilePath( FilePath.FileLocation.Disk, saveDialog.FileName );
                                m_notebook.NotifyCanvasStored();
                            } catch ( Exception e ) {
                                MessageBox.Show("Could not save file " + saveDialog.FileName  + " - Exception error message: " + e.Message );
                            } 
                        }
                    }
                    break;


                case ButtonType.Open: {
                        OpenFileDialog openDialog = new OpenFileDialog();
                        openDialog.FileName = "notebook.xml";
                        openDialog.Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*";
                        DialogResult openResult = openDialog.ShowDialog();
                        if (openResult == DialogResult.OK) {
                            XmlDocument doc = new XmlDocument();
                            try {
                                doc.Load( openDialog.FileName );
                            } catch (Exception) {
                                MessageBox.Show( "Error reading XML file " + openDialog.FileName );
                                return;
                            }

                            XmlNodeList vNotebook = doc.GetElementsByTagName( "Notebook" );
                            if (vNotebook.Count < 1) {
                                MessageBox.Show( "File " + openDialog.FileName + " contains more than one Notebook??" );
                                return;
                            }

                            XmlElement notebook_node = vNotebook[0] as XmlElement;
                            int nVersion = 0;
                            try {
                                string versionNumber = notebook_node.GetAttribute( "Version" );
                                nVersion = Int32.Parse( versionNumber );
                            } catch (Exception) { }

                            m_notebook.Restore( doc, notebook_node );
                            m_notebook.Path = new FilePath( FilePath.FileLocation.Disk, openDialog.FileName );
                        }
                    }
                    break;


                case ButtonType.AppendPage:
                    m_notebook.AppendPage(true);
                    break;

                case ButtonType.NextPage:
                    m_notebook.NextPage();
                    break;

                case ButtonType.PreviousPage:
                    m_notebook.PreviousPage();
                    break;

                case ButtonType.SelectPen:
                    m_notebook.SelectTool(VectorStrokeTool.ToolName);
                    TargetElement.ExternalNotifyElementChanged();
                    break;

                case ButtonType.SelectEraser:
                    m_notebook.SelectTool(VectorEraserTool.ToolName);
                    TargetElement.ExternalNotifyElementChanged();
                    break;

                case ButtonType.Undo:
                    m_notebook.CurrentPage.Undo();
                    break;

            }

        }
    }
}
