using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher {


    class VectorEraserTool : VectorTool {

        Blob m_eraserStamp;
        bool m_bErasing;
        EraserCursorRenderer m_renderer;

        public static readonly string ToolName = "EraserTool";

        public VectorEraserTool(VectorCanvas target)
            : base(target) 
        {
            m_eraserStamp = new Blob();
            m_eraserStamp.OuterRadius = 20;
            m_eraserStamp.InnerRadius = 10;

            //Cursor = new EraserCursor(target, m_eraserStamp);
            //Cursor.Resize( new Box2f( m_eraserStamp.OuterRadius*2 ) );

            m_renderer = new EraserCursorRenderer(this, m_eraserStamp);
        }


        public bool Erasing {
            get { return m_bErasing; }
        }


        public override string Name {
            get { return ToolName; }
        }


        override public InputCaptureType CaptureType {
            get { return InputCaptureType.FirstVertexOrNeverCapture; }
        }

        public override float GetCaptureScore(Stroke s) {
            if (TargetCanvas.ChildAreaBounds.Contains(s.First))
                return 1.0f;
            return 0.0f;
        }

        public override bool OnCaptureStrokeStart( Stroke s ) {
            if (base.OnCaptureStrokeStart( s )) {
                m_eraserStamp.Center = LastInputPosition;
                TargetCanvas.BeginErasing();
                TargetCanvas.Erase( m_eraserStamp );
                m_bErasing = true;
                return true;
            }
            return false;
        }

        public override bool OnCapturedStrokeUpdate(Stroke s) {
            if (base.OnCapturedStrokeUpdate( s )) {
                m_eraserStamp.Center = LastInputPosition;
                TargetCanvas.Erase( m_eraserStamp );
                return true;
            }
            return false;
        }

        public override bool OnCaptureStrokeEnd(Stroke s) {
            TargetCanvas.DoneErasing();
            m_bErasing = false;
            return true;
        }





        class EraserCursor : Widget {
            Blob m_eraserBlob;

            public EraserCursor(VectorCanvas canvas, Blob eraserBlob )
                : base(canvas) {
                m_eraserBlob = eraserBlob;
            }

            protected override void RenderWidget() {
                m_eraserBlob.Center = Bounds.Center;
                Circle2f eraserCircle = new Circle2f( m_eraserBlob.Center, m_eraserBlob.OuterRadius );
                RenderEngine.Get.DrawCircleOutline( eraserCircle, 1.0f, ColorRGBA.GREY );
            }
        }




        class EraserCursorRenderer : RenderingBehavior {
            Blob m_eraserBlob;
            VectorEraserTool m_targetTool;

            public EraserCursorRenderer(VectorEraserTool target, Blob eraserBlob) : base(target) {
                m_targetTool = target;
                m_eraserBlob = eraserBlob;
            }

            public override bool Enabled {
                get { return m_targetTool.Enabled; }
            }

            public override bool OnOverlayRender() {
                Circle2f eraserCircle = new Circle2f(m_targetTool.LastInputPosition, m_eraserBlob.OuterRadius);
                RenderEngine.Get.DrawCircleOutline(eraserCircle, 1.0f, ColorRGBA.GREY);
                return true;
            }
            public override bool OverlayRenderFlag {
                get { return true; }
            }


        }



    }

}
